/*
    JPC-RR: A x86 PC Hardware Emulator
    Release 1

    Copyright (C) 2007-2009 Isis Innovation Limited
    Copyright (C) 2009 H. Ilari Liusvaara

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Based on JPC x86 PC Hardware emulator,
    A project from the Physics Dept, The University of Oxford

    Details about original JPC can be found at:

    www-jpc.physics.ox.ac.uk

*/

package org.jpc.emulator.memory.codeblock;

/**
 * Supplies a stream of JPC instructions for compiling a codeblock.
 * @author Chris Dennis
 */
public interface InstructionSource
{
    /**
     * Advances to the next x86 instruction in the stream.
     * @return <code>true</code> if there are more instructions.
     */
    public boolean getNext();

    /**
     * Get the next microcode component of this x86 operation.
     * @return microcode value
     */
    public int getMicrocode();

    /**
     * Get the length of this x86 instruction in microcodes
     * @return length in microcodes
     */
    public int getLength();

    /**
     * Get the lenth of this x86 instruction in original x86 bytes.
     * @return length of this instruction in bytes.
     */
    public int getX86Length();

    /**
     * Reset to the start of the instruction source
     */
    public void reset();
}
